@@ -62,14 +62,13 @@ def jupyter_config_dir():
     """
 
     env = os.environ
-    home_dir = get_home_dir()
-
     if env.get('JUPYTER_NO_CONFIG'):
         return _mkdtemp_once('jupyter-clean-cfg')
 
     if env.get('JUPYTER_CONFIG_DIR'):
         return env['JUPYTER_CONFIG_DIR']
 
+    home_dir = get_home_dir()
     return pjoin(home_dir, '.jupyter')
 
 
@@ -168,7 +167,13 @@ def jupyter_path(*subdirs):
     # Next is environment or user, depending on the JUPYTER_PREFER_ENV_PATH flag
     user = [jupyter_data_dir()]
     if site.ENABLE_USER_SITE:
-        userdir = os.path.join(site.getuserbase(), 'share', 'jupyter')
+        # Check if site.getuserbase() exists to be compatible with virtualenv,
+        # which often does not have this method.
+        if hasattr(site, 'getuserbase'):
+            userbase = site.getuserbase()
+        else:
+            userbase = site.USER_BASE
+        userdir = os.path.join(userbase, 'share', 'jupyter')
         if userdir not in user:
             user.append(userdir)
 
@@ -231,7 +236,14 @@ def jupyter_config_path():
     # Next is environment or user, depending on the JUPYTER_PREFER_ENV_PATH flag
     user = [jupyter_config_dir()]
     if site.ENABLE_USER_SITE:
-        userdir = os.path.join(site.getuserbase(), 'etc', 'jupyter')
+        # Check if site.getuserbase() exists to be compatible with virtualenv,
+        # which often does not have this method.
+        if hasattr(site, 'getuserbase'):
+            userbase = site.getuserbase()
+        else:
+            userbase = site.USER_BASE
+
+        userdir = os.path.join(userbase, 'etc', 'jupyter')
         if userdir not in user:
             user.append(userdir)
 
@@ -864,6 +876,7 @@ def secure_write(fname, binary=False):
         Indicates that the file is binary
     """
     mode = 'wb' if binary else 'w'
+    encoding = None if binary else 'utf-8'
     open_flag = os.O_CREAT | os.O_WRONLY | os.O_TRUNC
     try:
         os.remove(fname)
@@ -885,7 +898,7 @@ def secure_write(fname, binary=False):
             open_flag = os.O_WRONLY | os.O_TRUNC
             win32_restrict_file_to_user(fname)
 
-    with os.fdopen(os.open(fname, open_flag, 0o0600), mode) as f:
+    with os.fdopen(os.open(fname, open_flag, 0o0600), mode, encoding=encoding) as f:
         if os.name != 'nt':
             # Enforce that the file got the requested permissions before writing
             file_mode = get_file_mode(fname)
